/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kylin.background 3.0 as KWin
import QtGraphicalEffects 1.0

Repeater {
    id : allroot
    model : screencnt
    focus: true
    Keys.enabled: true
    property int screencnt : multitaskViewModel.logicalScreenCount

    function closeMultitaskView() {
        console.log("closeMultitaskView() !")
        multitaskViewManager.close()
    }

    Keys.onPressed : {
        if (event.key === Qt.Key_Escape) {
            closeMultitaskView();
            return;
        }
        if (event.modifiers & Qt.MetaModifier) {
            if (event.key === Qt.Key_Tab) {
                closeMultitaskView();
                return;
            }
        }

    }

    Rectangle {
        id : rootWindow
        x : rootWindow.screenRect.x
        y : rootWindow.screenRect.y
        width : rootWindow.screenRect.width
        height : rootWindow.screenRect.height
        property var isWlcom : multitaskViewManager.isWlcom()
        property var isVerticalLayout : multitaskViewManager.isMultitaskViewVerticalLayout()
        property int highlightDesktop : multitaskViewModel.highlightDesktop
        property var screenRect : multitaskViewModel.getFullScreenGeometry(highlightDesktop, index)
        property var scaleFactor : screenRect.width > screenRect.height ? screenRect.height / 1080 : screenRect.width / 1080
        property var screenId : multitaskViewModel.getOutputUuid(index)
        property bool inputMode : multitaskViewModel.inputMode
        property var currentTheme : multitaskViewModel.getUKUIStyleName()
        property var closeButtonGeneralHoverColor
        property var closeButtonGeneralClickColor
        property var desktopCloseButtonDefaultColor
        property var closeButtonGeneralImage
        property var createDesktopButtonReceiveColor
        property var createDesktopButtonDefaultColor
        property var desktopMaskReceiveColor
        property var desktopCaptionColor
        property var desktopAreaColor
        property var scrollDefaultColor
        property var scrollHoverColor
        property var hoverBorderColor : "#FFFFFF"
        property var tabFocusBorderColor : "#FFFFFF"
        // 预览图布局区域 y 轴起始高度
        property var scrollViewYStartPoint : Math.floor(13 * scaleFactor)
        // 预览图布局区域距离左侧和右侧的间距
        property int scrollViewLeftSpace : isVerticalLayout ? 32 : 0
        property var iconAndCloseButtonHeight : Math.floor(24 * scaleFactor)
        property var appBorderWidth : Math.floor(4 * scaleFactor)
        property var appRadius : Math.floor(multitaskViewModel.getRadius("Max_Radius") * scaleFactor)
        property var spacingWidth : Math.floor(2 * scaleFactor)
        property var appThumbnailMargin : appBorderWidth + spacingWidth
        property var workspaceBorderWidth : Math.floor(3 * scaleFactor)
        property var workspaceRadius : Math.floor(multitaskViewModel.getRadius("Normal_Radius") * scaleFactor)
        property var thumbRectangleHeight : isVerticalLayout ? Math.floor(screenRect.height * 5 / 22) : Math.floor(screenRect.height * 5 / 23)
        property var previewWindowHeight : thumbRectangleHeight + 2 * appBorderWidth + 3 * spacingWidth + iconAndCloseButtonHeight
        // 工作区窗口缩放比
        property double desktopScale : isVerticalLayout ? 16 / 135 : 57 / 480
        // 工作区窗口的高度和宽度
        property int desktopThumbnailWidth : screenRect.width * desktopScale
        property int desktopThumbnailHeight : screenRect.height * desktopScale
        property ListModel appWindowModel : ListModel{}
        property ListModel desktopWindowModel : ListModel{}
        //水平布局方向
        property bool isLayoutLeftToRight: multitaskViewManager.isLayoutLeftToRight()


        KWin.DesktopBackground {
            id : wholeDesktopBackground
            width : rootWindow.width
            height : rootWindow.height
            blurAlpha : 0.7
        }

        function setItemColor() {
            if (rootWindow.currentTheme === "ukui-default" || rootWindow.currentTheme === "ukui-white"
                    || rootWindow.currentTheme === "ukui" || rootWindow.currentTheme === "ukui-light") {
                rootWindow.createDesktopButtonReceiveColor = Qt.rgba(0, 0, 0, 0.25)
                rootWindow.createDesktopButtonDefaultColor = Qt.rgba(0, 0, 0, 0.15)
                rootWindow.desktopMaskReceiveColor = Qt.rgba(0, 0, 0, 0.25)
                rootWindow.desktopCaptionColor = Qt.rgba(0, 0, 0, 1)
                rootWindow.desktopAreaColor = Qt.rgba(245/255, 245/255, 245/255, 0.75)
                rootWindow.scrollDefaultColor = Qt.rgba(0, 0, 0, 0.15)
                rootWindow.scrollHoverColor = Qt.rgba(0, 0, 0, 0.25)
                rootWindow.tabFocusBorderColor = "#2c73c8"
                desktopCloseButtonDefaultColor = Qt.rgba(38/255, 38/255, 38/255, 0.15)
                closeButtonGeneralHoverColor = Qt.rgba(1, 1, 1, 1)
                closeButtonGeneralClickColor = Qt.rgba(1, 1, 1, 0.75)
                closeButtonGeneralImage = "qrc:/images/application-exit-symbolic.svg"
            } else {
                rootWindow.createDesktopButtonReceiveColor = isVerticalLayout ? Qt.rgba(0, 0, 0, 0.1) : Qt.rgba(1, 1, 1, 0.25)
                rootWindow.createDesktopButtonDefaultColor = Qt.rgba(1, 1, 1, 0.15)
                rootWindow.desktopMaskReceiveColor = Qt.rgba(1, 1, 1, 0.25)
                rootWindow.desktopCaptionColor = Qt.rgba(1, 1, 1, 1)
                rootWindow.desktopAreaColor = Qt.rgba(35/255, 36/255, 38/255, 0.75)
                rootWindow.scrollDefaultColor = Qt.rgba(1, 1, 1, 0.15)
                rootWindow.scrollHoverColor = Qt.rgba(1, 1, 1, 0.25)
                rootWindow.tabFocusBorderColor = "#5fa6fb"
                desktopCloseButtonDefaultColor = Qt.rgba(1, 1, 1, 0.15)
                closeButtonGeneralHoverColor =  "#37373B"
                closeButtonGeneralClickColor = Qt.rgba(75/255, 75/255, 79/255, 1)
                closeButtonGeneralImage = "qrc:/images/application-exit-symbolic-white.svg"
            }
        }

        FocusScope {
            id: mainView
            anchors.fill: parent
            focus: true
            width: parent.width
            height: parent.height
            property var screenIndex : index

            Loader {
                id: viewLoader
                sourceComponent: isVerticalLayout ? verticalLayoutView : horizontalLayoutView
                anchors.fill: parent
            }

            Component {
                id: verticalLayoutView
                Rectangle {
                    color: "transparent"
                    AppArea {
                        id : appArea1
                        anchors.left : parent.left
                        anchors.right : parent.right
                        anchors.top : parent.top
                        height : Math.floor(mainView.height * 111 / 135)
                        width: parent.width

                        Component.onCompleted : {
                            console.log("AppArea end! ! !")
                        }
                    }
                    DesktopArea {
                        id : desktopArea1
                        anchors.left : parent.left
                        anchors.right : parent.right
                        anchors.top : appArea1.bottom
                        anchors.bottom : parent.bottom
                    }

                    Component.onCompleted : {
                        console.log("verticalLayoutView end! ! !")
                    }
                }
            }

            Component {
                id: horizontalLayoutView
                Rectangle {
                    color: "transparent"
                    AppArea {
                        id : appArea2
                        anchors.left : parent.left
                        anchors.bottom : parent.bottom
                        anchors.top : parent.top
                        height: parent.height
                        width : Math.floor(mainView.width * 410 / 480)

                        Component.onCompleted : {
                            console.log("MasterAppArea end! ! !")
                        }
                    }

                    MasterDesktopArea {
                        id : desktopArea2
                        anchors.left : appArea2.right
                        anchors.right : parent.right
                        anchors.top : parent.top
                        anchors.bottom : parent.bottom
                    }

                    Component.onCompleted : {
                        console.log("horizontalLayoutView end! ! !")
                    }
                }
            }
        }

        Component.onCompleted : {
            setItemColor()
        }
    }
}
